ColorConfiguratorDialog = {
	CONTROLS = {
		"multiMaterialType",
		"textInputRed",
		"textInputGreen",
		"textInputBlue",
		"sliderRed",
		"sliderGreen",
		"sliderBlue",
		"sceneRender",
		"tooltipLayout",
		"textHex",
		"lookGlyph",
		"zoomGlyph",
		"loadingAnimation",
		"saveButton"
	}
}

local ColorConfiguratorDialog_mt = Class(ColorConfiguratorDialog, MessageDialog)

function ColorConfiguratorDialog.new(target, custom_mt, xmlFilename, colorConfigurator)
	local self = MessageDialog.new(target, custom_mt or ColorConfiguratorDialog_mt)

	self:registerControls(ColorConfiguratorDialog.CONTROLS)

	self.colorConfigurator = colorConfigurator
	self.colorConfiguratorPickerDialog = colorConfigurator.colorConfiguratorPickerDialog

	self.materials = self:loadMaterials(xmlFilename)
	self.colors = {1, 1, 1, 0}

	self.showHex = false
	self.showGlyphs = true
	self.showLoadingAnimation = false

	self.roundOnScroll = false

	self.lastInputHelpMode = nil
	self.sharedLoadRequestId = nil

	return self
end

function ColorConfiguratorDialog:loadMaterials(xmlFilename)
	local materials = {}
	local xmlFile = XMLFile.load("materialsXML", xmlFilename)

	if xmlFile ~= nil then
		xmlFile:iterate("materials.material", function (_, key)
			local name = xmlFile:getString(key .. "#name")
			local shaderId = xmlFile:getInt(key .. "#shaderId")

			if name ~= nil and name:len() > 0 and shaderId ~= nil then
				materials[#materials + 1] = {
					name = name,
					shaderId = shaderId
				}
			end
		end)

		xmlFile:delete()
	end

	return materials
end

function ColorConfiguratorDialog:delete()
	if self.sharedLoadRequestId ~= nil then
		g_i3DManager:releaseSharedI3DFile(self.sharedLoadRequestId)
		self.sharedLoadRequestId = nil
	end

	ColorConfiguratorDialog:superClass().delete(self)
end

function ColorConfiguratorDialog:onGuiSetupFinished()
	ColorConfiguratorDialog:superClass().onGuiSetupFinished(self)

	local materialsTexts = {}

	for i, material in pairs(self.materials) do
		table.insert(materialsTexts, string.format("%s (%d)", g_i18n:getText(string.format("colorConfigurator_material_%s", material.name)), i))
	end

	self.multiMaterialType:setTexts(materialsTexts)

	for i, textInputElement in pairs({self.textInputRed, self.textInputGreen, self.textInputBlue}) do
		textInputElement:setText(tostring(self.colors[i]))
		textInputElement.customFocusSample = nil
	end

	for _, sliderElement in pairs({self.sliderRed, self.sliderGreen, self.sliderBlue}) do
		sliderElement.steps = 0

		sliderElement.setValue = Utils.overwrittenFunction(sliderElement.setValue, function(slider, superFunc, newValue, doNotUpdateDataElement, immediateMode)
			if not immediateMode and not slider.mouseDown and (Input.isMouseButtonPressed(Input.MOUSE_BUTTON_WHEEL_UP) or Input.isMouseButtonPressed(Input.MOUSE_BUTTON_WHEEL_DOWN)) then
				slider.steps = slider.steps + 1

				if slider.steps >= 2 then
					slider.steps = 0

					return false
				end

				if self.roundOnScroll then
					newValue = MathUtil.round(newValue, 2)
				end
			end

			local stepSizeDefault = slider.stepSize
			slider.stepSize = stepSizeDefault * 0.01

			local retValue = superFunc(slider, newValue, doNotUpdateDataElement, immediateMode)

			slider.stepSize = stepSizeDefault

			return retValue
		end)
	end

	self.sceneRender.filename = NetworkUtil.convertFromNetworkFilename(self.sceneRender.filename)

	self.buttonBox = self.lookGlyph.parent

	self.textHex:setVisible(self.showHex)
	self.buttonBox:setVisible(self.showGlyphs)
end

function ColorConfiguratorDialog:sceneRenderI3DLoaded(i3dNode, failedReason, args)
end

function ColorConfiguratorDialog:update(dt)
	ColorConfiguratorDialog:superClass().update(self, dt)

	if self.showGlyphs then
		local inputHelpMode = g_inputBinding:getInputHelpMode()

		if inputHelpMode ~= self.lastInputHelpMode then
			self.lastInputHelpMode = inputHelpMode

			self.buttonBox:setVisible(inputHelpMode ~= GS_INPUT_HELP_MODE_GAMEPAD)

			self.lookGlyph:setActions({
				InputAction.AXIS_LOOK_LEFTRIGHT_DRAG,
				InputAction.AXIS_LOOK_UPDOWN_DRAG
			})

			self.zoomGlyph:setActions({
				InputAction.AXIS_MAP_ZOOM_IN,
				InputAction.AXIS_MAP_ZOOM_OUT
			})
		end
	end
end

function ColorConfiguratorDialog:onClickOk()
	self:close()
	self.colorConfigurator:setColor(self.colors)
end

function ColorConfiguratorDialog:onClickSave()
	g_gui:showTextInputDialog({
		defaultText = g_i18n:getText("colorConfigurator_ui_defaultColorName"),
		dialogPrompt = g_i18n:getText("colorConfigurator_ui_enterColorName"),
		imePrompt = g_i18n:getText("colorConfigurator_ui_enterColorName"),
		confirmText = g_i18n:getText("button_save"),
		maxCharacters = 30,
		callback = function (result, yes)
			if yes then
				local storeItem = self.colorConfiguratorPickerDialog.target.storeItem
				local configName = self.colorConfiguratorPickerDialog.args.configName
				local r, g, b, mat = unpack(self.colors)

				local data = {
					name = result,
					material = mat,
					color = {r, g, b },
					isAddConfig = true
				}

				if storeItem.bundleInfo ~= nil then
					for _, bundleItem in ipairs(storeItem.bundleInfo.bundleItems) do
						self.colorConfigurator:addAdditionalConfig(bundleItem.xmlFilename, configName, data, true)
					end
				end

				self.colorConfigurator:addAdditionalConfig(storeItem.xmlFilename, configName, data)

				g_gui:showInfoDialog({
					text = string.format(g_i18n:getText("colorConfigurator_ui_colorAddedToList"), data.name),
					dialogType = DialogElement.TYPE_INFO,
					callback = function ()
						self:onClickBack()
					end
				})
			end
		end,
	})
end

function ColorConfiguratorDialog:onClickBack()
	ColorConfiguratorDialog:superClass().onClickBack(self)

	local colorConfiguratorPickerDialog = self.colorConfiguratorPickerDialog

	colorConfiguratorPickerDialog.disableOpenSound = true
	g_gui:showDialog(colorConfiguratorPickerDialog.name)
	colorConfiguratorPickerDialog:setColors(unpack(colorConfiguratorPickerDialog.previousColors))
	colorConfiguratorPickerDialog.disableOpenSound = false
end

function ColorConfiguratorDialog:onOpen()
	ColorConfiguratorDialog:superClass().onOpen(self)

	if self.showLoadingAnimation then
		self.loadingAnimation:setVisible(true)
	end

	if FocusManager:getFocusedElement() == g_gui.guis[self.name] then
		self:setSoundSuppressed(true)
		FocusManager:setFocus(self.multiMaterialType)
		self:setSoundSuppressed(false)
	end

	if self.sharedLoadRequestId == nil then
		self.sharedLoadRequestId = g_i3DManager:loadSharedI3DFileAsync(self.sceneRender.filename, false, false, self.sceneRenderI3DLoaded, self, nil)
	end

	self.sceneRender:createScene()
end

function ColorConfiguratorDialog:onClose()
	ColorConfiguratorDialog:superClass().onClose(self)

	if self.tooltipLayout:getIsVisible() then
		self.tooltipLayout:setVisible(false)
	end

	self.sceneRender:destroyScene()
end

function ColorConfiguratorDialog:onRenderLoad(scene, overlay)
	self:updateSceneRenderShaderParameters()

	if self.loadingAnimation:getIsVisible() then
		self.loadingAnimation:setVisible(false)
	end
end

function ColorConfiguratorDialog:onClickMaterialType(state)
	self.colors[4] = self.materials[state].shaderId
	self:updateSceneRenderShaderParameters()
	self:hideTooltipLayout()
end

function ColorConfiguratorDialog:onLeaveText(element)
	self:verifyInputText(element)
end

function ColorConfiguratorDialog:onTextChangedRed(element)
	self:onTextChanged(self.sliderRed, 1, element.text)
end

function ColorConfiguratorDialog:onTextChangedGreen(element)
	self:onTextChanged(self.sliderGreen, 2, element.text)
end

function ColorConfiguratorDialog:onTextChangedBlue(element)
	self:onTextChanged(self.sliderBlue, 3, element.text)
end

function ColorConfiguratorDialog:onChangedSliderRed(value)
	self:onSliderChanged(self.textInputRed, 1, value)
end

function ColorConfiguratorDialog:onChangedSliderGreen(value)
	self:onSliderChanged(self.textInputGreen, 2, value)
end

function ColorConfiguratorDialog:onChangedSliderBlue(value)
	self:onSliderChanged(self.textInputBlue, 3, value)
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextRed(unicode)
	return self:onIsUnicodeAllowed(self.textInputRed.text, unicode)
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextGreen(unicode)
	return self:onIsUnicodeAllowed(self.textInputGreen.text, unicode)
end

function ColorConfiguratorDialog:onIsUnicodeAllowedTextBlue(unicode)
	return self:onIsUnicodeAllowed(self.textInputBlue.text, unicode)
end

function ColorConfiguratorDialog:onButtonPressed(element, dismissal)
	self:verifyInputText(element)
end

function ColorConfiguratorDialog:onTextChanged(sliderElement, colorId, value)
	local number = tonumber(value) or 0

	self.colors[colorId] = number

	if sliderElement.currentValue ~= number then
		sliderElement:setValue(number)
	end

	self:hideTooltipLayout()
end

function ColorConfiguratorDialog:onSliderChanged(textInputElement, colorId, value)
	local text = tostring(value) or "0"

	if textInputElement.text ~= "" and tonumber(textInputElement.text) ~= value then
		textInputElement:setText(text)
	end

	self:updateSceneRenderShaderParameters()
	self:playSample(GuiSoundPlayer.SOUND_SAMPLES.HOVER)
end

function ColorConfiguratorDialog:onIsUnicodeAllowed(inputText, unicode)
	local text = inputText .. unicodeToUtf8(unicode)
	local number = tonumber(text)

	return unicode ~= Input.KEY_space and text ~= "1." and number ~= nil and not MathUtil.getIsOutOfBounds(number, 0, 1)
end

function ColorConfiguratorDialog:setValues(colors, material, hasMaterial)
	local r, g, b = unpack(colors)

	self.textInputRed:setText(tostring(r))
	self.textInputGreen:setText(tostring(g))
	self.textInputBlue:setText(tostring(b))

	local state = 1

	for i, mat in pairs(self.materials) do
		if mat.shaderId == material then
			state = i
		end
	end

	self.multiMaterialType:setState(state, true)

	if not hasMaterial then
		self.tooltipLayout:setVisible(true)
	end

	self:updateSceneRenderShaderParameters()
end

function ColorConfiguratorDialog:verifyInputText(textInputElement)
	textInputElement:setText(tostring(tonumber(textInputElement.text) or 0))
end

function ColorConfiguratorDialog:updateSceneRenderShaderParameters()
	local r, g, b, mat = unpack(self.colors)

	if self.textHex:getIsVisible() then
		self.textHex:setText(string.format("hex: #%02x%02x%02x", math.floor(r * 255), math.floor(g * 255), math.floor(b * 255)))
	end

	self:setSaveButtonDisabled(r, g, b, mat)
	self.sceneRender:updateShaderParameters(r, g, b, mat)
end

function ColorConfiguratorDialog:hideTooltipLayout()
	if self.isOpen then
		if self.tooltipLayout:getIsVisible() then
			self.tooltipLayout:setVisible(false)
		end
	end
end

function ColorConfiguratorDialog:setSaveButtonDisabled(r, g, b, material)
	if not self.isOpen then
		return
	end

	local alreadyExists = false
	local storeItem = self.colorConfiguratorPickerDialog.target.storeItem
	local configName = self.colorConfiguratorPickerDialog.args.configName
	local baseConfigs = storeItem.configurations[configName]

	for _, config in pairs(baseConfigs) do
		if not config.isCustomConfig then
			local configMaterial = config.material
			local configColor = config.color

			if material == configMaterial and configColor[1] == r and configColor[2] == g and configColor[3] == b then
				alreadyExists = true
				break
			end
		end
	end

	local addConfigsCount = 0

	if not alreadyExists then
		local filename = storeItem.xmlFilename
		local addConfigs = self.colorConfigurator:getAdditionalConfigs(filename, configName)

		if addConfigs ~= nil then
			addConfigsCount = #addConfigs

			for _, addColor in ipairs(addConfigs) do
				local configMaterial = addColor.material
				local configColor = addColor.color

				if material == configMaterial and configColor[1] == r and configColor[2] == g and configColor[3] == b then
					alreadyExists = true
					break
				end
			end
		end
	end

	self.saveButton:setDisabled(alreadyExists or #baseConfigs + addConfigsCount >= ColorConfigurator.MAX_CONFIGS)
end